class LSystem {
  
  protected char[] l_alphabet = {'F', '+', '-', '[', ']'};
  protected String[] l_rules = {"F[+F]F[-F]F", "+", "-", "[", "]"};
  
  protected String l_axiom = "F";
  protected String l_tree = "";
  
  LSystem() {
    
  }
    
  public void generate (int depth) {
    l_tree = l_axiom;
    
    // cache the lengths of each rule
    int[] ruleLengths = new int[l_alphabet.length]; 
    for (int i = 0; i < l_alphabet.length; i++) {
      ruleLengths[i] = l_rules[i].length();
    }
    
    // iterate over the tree - calculate the new tree length and then generate tree from it
      
    for (int i = 0; i < depth; i++) {
      
      int newLength = 0;
      int oldLength = l_tree.length();
      for (int j = 0; j < oldLength; j++) {
        char c = l_tree.charAt(j);
        for (int k = 0; k < l_alphabet.length; k++) {
          if (c == l_alphabet[k]) {
            newLength += ruleLengths[k];
          }
        }
      }
      
      // generate new tree
      StringBuffer newTree = new StringBuffer(newLength);
      
      for (int j = 0; j < oldLength; j++) {
        char c = l_tree.charAt(j);
        for (int k = 0; k < l_alphabet.length; k++) {
          if (c == l_alphabet[k]) {
            newTree.append(l_rules[k]);
            break;
          }
        }
      }
      // save the new tree
      l_tree = newTree.toString();
    }
    println("new tree:" + l_tree);
  }
  
  public String getTree() {
    return l_tree;
  }
  
  public String getRule (int index) {
    if (0 <= index && index < l_rules.length) {
      return l_rules[index];
    }
    return "";
  }
  
  public void reset() {
    l_alphabet = new char [] {'F', '+', '-', '[', ']'};
    l_rules = new String [] {"F[+F]F[-F]F", "+", "-", "[", "]"};
    l_axiom = "F";
  }
  
  public void setAxiom (String newAxiom) {
    l_axiom = newAxiom; // TODO: validate brackets  and chars
  }
  
  public void setAlphabet(char [] newAlphabet) {
    l_alphabet = newAlphabet;
  }
  
  public void setRules(String [] newRules) {
    l_rules = newRules; // TODO: check length?
  }
  
  public void setRule(int index, String rule) {
    if (0 <= index && index < l_rules.length) {
      l_rules[index] = rule;
    }
  }
}
