class LTree extends LSystem {
  
  private float t_distance = 50;
  private float t_angle = 60;
  private float t_scaleFactor = .7;

  LTree () {
    setInit();
  }
  
  public void setInit () {
    l_axiom = "F";
    l_alphabet = new char [] {'F', '+', '-', '[', ']', 'z', 'a', 'B', 'C'};
    l_rules = new String [] {"F[+F][-F][aF][zF]", "+", "-", "[", "]", "z", "a", "B", "Ca+F"};
  }

  public void drawTree() {
    for (int i = 0; i<l_tree.length(); i++) {
      switch (l_tree.charAt(i)) {
        case 'F':
        case 'B':
          float newhue = float(i)/l_tree.length()*255;
          fill(color(newhue, 255, 255));
          translate(0,-t_distance/2,0);
          box(t_distance/4,t_distance,t_distance/4);
          translate(0,-t_distance/2,0);
          break;
        case '-':
          rotateX(radians(-t_angle));
          break;
        case '+':
          rotateX(radians(t_angle));
          break;
        case 'z':
          rotateZ(radians(t_angle));
          break;
        case 'a':
          rotateZ(radians(-t_angle));
          break;
        case '[':
          pushMatrix();  
          t_distance = t_distance * t_scaleFactor;  
          break;  
        case ']':
          popMatrix();    
          t_distance = t_distance / t_scaleFactor;
          break;
      }
    }
  }
  
  public void setAngle (float newAngle) {
    t_angle = constrain(newAngle, -360, 360);
  }
  
  public void setScale (float newScale) {
    t_scaleFactor = newScale;
  }
}
